import * as React from "react";
import { Session } from ".";
import { ProviderType } from "./providers";
import { CtxOrReq } from "./lib/client";
export interface NextAuthClientConfig {
    baseUrl: string;
    basePath: string;
    baseUrlServer: string;
    basePathServer: string;
    /** Stores last session response */
    _session?: Session | null | undefined;
    /** Used for timestamp since last sycned (in seconds) */
    _lastSync: number;
    /**
     * Stores the `SessionProvider`'s session update method to be able to
     * trigger session updates from places like `signIn` or `signOut`
     */
    _getSession: (...args: any[]) => any;
}
export declare type SessionContextValue<R extends boolean = false> = R extends true ? {
    data: Session;
    status: "authenticated";
} | {
    data: null;
    status: "loading";
} : {
    data: Session;
    status: "authenticated";
} | {
    data: null;
    status: "unauthenticated" | "loading";
};
export interface UseSessionOptions<R extends boolean> {
    required: R;
    /** Defaults to `signIn` */
    onUnauthenticated?: () => void;
}
/**
 * React Hook that gives you access
 * to the logged in user's session data.
 *
 * [Documentation](https://next-auth.js.org/getting-started/client#usesession)
 */
export declare function useSession<R extends boolean>(options: UseSessionOptions<R>): SessionContextValue<R> | {
    readonly data: Session | null;
    readonly status: "loading";
};
export declare type GetSessionParams = CtxOrReq & {
    event?: "storage" | "timer" | "hidden" | string;
    triggerEvent?: boolean;
    broadcast?: boolean;
};
export declare function getSession(params?: GetSessionParams): Promise<Session | null>;
/**
 * Returns the current Cross Site Request Forgery Token (CSRF Token)
 * required to make POST requests (e.g. for signing in and signing out).
 * You likely only need to use this if you are not using the built-in
 * `signIn()` and `signOut()` methods.
 *
 * [Documentation](https://next-auth.js.org/getting-started/client#getcsrftoken)
 */
export declare function getCsrfToken(params?: CtxOrReq): Promise<string | undefined>;
export interface ClientSafeProvider {
    id: string;
    name: string;
    type: ProviderType;
    signinUrl: string;
    callbackUrl: string;
}
export declare type RedirectableProvider = "email" | "credentials";
/**
 * It calls `/api/auth/providers` and returns
 * a list of the currently configured authentication providers.
 * It can be useful if you are creating a dynamic custom sign in page.
 *
 * [Documentation](https://next-auth.js.org/getting-started/client#getproviders)
 */
export declare function getProviders(): Promise<Record<RedirectableProvider, ClientSafeProvider> | null>;
export interface SignInOptions extends Record<string, unknown> {
    /**
     * Defaults to the current URL.
     * @docs https://next-auth.js.org/getting-started/client#specifying-a-callbackurl
     */
    callbackUrl?: string;
    /** @docs https://next-auth.js.org/getting-started/client#using-the-redirect-false-option */
    redirect?: boolean;
}
export interface SignInResponse {
    error: string | undefined;
    status: number;
    ok: boolean;
    url: string | null;
}
/** Match `inputType` of `new URLSearchParams(inputType)` */
export declare type SignInAuthorisationParams = string | string[][] | Record<string, string> | URLSearchParams;
/**
 * Client-side method to initiate a signin flow
 * or send the user to the signin page listing all possible providers.
 * Automatically adds the CSRF token to the request.
 *
 * [Documentation](https://next-auth.js.org/getting-started/client#signin)
 */
export declare function signIn<P extends RedirectableProvider | undefined = undefined>(provider?: RedirectableProvider, options?: SignInOptions, authorizationParams?: SignInAuthorisationParams): Promise<P extends RedirectableProvider ? SignInResponse | undefined : undefined>;
/** @docs https://next-auth.js.org/getting-started/client#using-the-redirect-false-option-1 */
export interface SignOutResponse {
    url: string;
}
export interface SignOutParams<R extends boolean = true> {
    /** @docs https://next-auth.js.org/getting-started/client#specifying-a-callbackurl-1 */
    callbackUrl?: string;
    /** @docs https://next-auth.js.org/getting-started/client#using-the-redirect-false-option-1 */
    redirect?: R;
}
/**
 * Signs the user out, by removing the session cookie.
 * Automatically adds the CSRF token to the request.
 *
 * [Documentation](https://next-auth.js.org/getting-started/client#signout)
 */
export declare function signOut<R extends boolean = true>(options?: SignOutParams<R>): Promise<R extends true ? undefined : SignOutResponse>;
/** @docs: https://next-auth.js.org/getting-started/client#options */
export interface SessionProviderProps {
    children: React.ReactNode;
    session?: Session | null;
    baseUrl?: string;
    basePath?: string;
    /**
     * The amount of time (in seconds) after a session should be considered stale.
     * If set to `0` (default), the session will never be re-fetched.
     */
    staleTime?: number;
    /**
     * A time interval (in seconds) after which the session will be re-fetched.
     * If set to `0` (default), the session is not polled.
     */
    refetchInterval?: number;
}
/**
 * Provider to wrap the app in to make session data available globally.
 * Can also be used to throttle the number of requests to the endpoint
 * `/api/auth/session`.
 *
 * [Documentation](https://next-auth.js.org/getting-started/client#sessionprovider)
 */
export declare function SessionProvider(props: SessionProviderProps): JSX.Element;
