var _a, _b, _c, _d;
import { jsx as _jsx } from "react/jsx-runtime";
// Note about signIn() and signOut() methods:
//
// On signIn() and signOut() we pass 'json: true' to request a response in JSON
// instead of HTTP as redirect URLs on other domains are not returned to
// requests made using the fetch API in the browser, and we need to ask the API
// to return the response as a JSON object (the end point still defaults to
// returning an HTTP response with a redirect for non-JavaScript clients).
//
// We use HTTP POST requests with CSRF Tokens to protect against CSRF attacks.
import * as React from "react";
import _logger, { proxyLogger } from "./lib/logger";
import parseUrl from "./lib/parse-url";
import { BroadcastChannel, apiBaseUrl, fetchData, now, } from "./lib/client";
// This behaviour mirrors the default behaviour for getting the site name that
// happens server side in server/index.js
// 1. An empty value is legitimate when the code is being invoked client side as
//    relative URLs are valid in that context and so defaults to empty.
// 2. When invoked server side the value is picked up from an environment
//    variable and defaults to 'http://localhost:3000'.
const __NEXTAUTH = {
    baseUrl: parseUrl((_a = process.env.NEXTAUTH_URL) !== null && _a !== void 0 ? _a : process.env.VERCEL_URL).baseUrl,
    basePath: parseUrl(process.env.NEXTAUTH_URL).basePath,
    baseUrlServer: parseUrl((_c = (_b = process.env.NEXTAUTH_URL_INTERNAL) !== null && _b !== void 0 ? _b : process.env.NEXTAUTH_URL) !== null && _c !== void 0 ? _c : process.env.VERCEL_URL).baseUrl,
    basePathServer: parseUrl((_d = process.env.NEXTAUTH_URL_INTERNAL) !== null && _d !== void 0 ? _d : process.env.NEXTAUTH_URL).basePath,
    _lastSync: 0,
    _session: undefined,
    _getSession: () => { },
};
const broadcast = BroadcastChannel();
const logger = proxyLogger(_logger, __NEXTAUTH.basePath);
const SessionContext = React.createContext(undefined);
/**
 * React Hook that gives you access
 * to the logged in user's session data.
 *
 * [Documentation](https://next-auth.js.org/getting-started/client#usesession)
 */
export function useSession(options) {
    // @ts-expect-error Satisfy TS if branch on line below
    const value = React.useContext(SessionContext);
    if (!value && process.env.NODE_ENV !== "production") {
        throw new Error("useSession must be wrapped in a SessionProvider");
    }
    const { required, onUnauthenticated } = options !== null && options !== void 0 ? options : {};
    const requiredAndNotLoading = required && value.status === "unauthenticated";
    React.useEffect(() => {
        if (requiredAndNotLoading) {
            const url = `/api/auth/signin?${new URLSearchParams({
                error: "SessionRequired",
                callbackUrl: window.location.href,
            })}`;
            if (onUnauthenticated)
                onUnauthenticated();
            else
                window.location.replace(url);
        }
    }, [requiredAndNotLoading, onUnauthenticated]);
    if (requiredAndNotLoading) {
        return { data: value.data, status: "loading" };
    }
    return value;
}
export async function getSession(params) {
    var _a;
    const session = await fetchData("session", __NEXTAUTH, logger, params);
    if ((_a = params === null || params === void 0 ? void 0 : params.broadcast) !== null && _a !== void 0 ? _a : true) {
        broadcast.post({ event: "session", data: { trigger: "getSession" } });
    }
    return session;
}
/**
 * Returns the current Cross Site Request Forgery Token (CSRF Token)
 * required to make POST requests (e.g. for signing in and signing out).
 * You likely only need to use this if you are not using the built-in
 * `signIn()` and `signOut()` methods.
 *
 * [Documentation](https://next-auth.js.org/getting-started/client#getcsrftoken)
 */
export async function getCsrfToken(params) {
    const response = await fetchData("csrf", __NEXTAUTH, logger, params);
    return response === null || response === void 0 ? void 0 : response.csrfToken;
}
/**
 * It calls `/api/auth/providers` and returns
 * a list of the currently configured authentication providers.
 * It can be useful if you are creating a dynamic custom sign in page.
 *
 * [Documentation](https://next-auth.js.org/getting-started/client#getproviders)
 */
export async function getProviders() {
    return await fetchData("providers", __NEXTAUTH, logger);
}
/**
 * Client-side method to initiate a signin flow
 * or send the user to the signin page listing all possible providers.
 * Automatically adds the CSRF token to the request.
 *
 * [Documentation](https://next-auth.js.org/getting-started/client#signin)
 */
export async function signIn(provider, options, authorizationParams) {
    var _a;
    const { callbackUrl = window.location.href, redirect = true } = options !== null && options !== void 0 ? options : {};
    const baseUrl = apiBaseUrl(__NEXTAUTH);
    const providers = await getProviders();
    if (!providers) {
        window.location.replace(`${baseUrl}/error`);
        return;
    }
    if (!provider || !(provider in providers)) {
        window.location.replace(`${baseUrl}/signin?${new URLSearchParams({ callbackUrl })}`);
        return;
    }
    const isCredentials = providers[provider].type === "credentials";
    const isEmail = providers[provider].type === "email";
    const isSupportingReturn = isCredentials || isEmail;
    const signInUrl = `${baseUrl}/${isCredentials ? "callback" : "signin"}/${provider}`;
    const _signInUrl = `${signInUrl}?${new URLSearchParams(authorizationParams)}`;
    const res = await fetch(_signInUrl, {
        method: "post",
        headers: {
            "Content-Type": "application/x-www-form-urlencoded",
        },
        // @ts-expect-error
        body: new URLSearchParams({
            ...options,
            csrfToken: await getCsrfToken(),
            callbackUrl,
            json: true,
        }),
    });
    const data = await res.json();
    if (redirect || !isSupportingReturn) {
        const url = (_a = data.url) !== null && _a !== void 0 ? _a : callbackUrl;
        window.location.replace(url);
        // If url contains a hash, the browser does not reload the page. We reload manually
        if (url.includes("#"))
            window.location.reload();
        return;
    }
    const error = new URL(data.url).searchParams.get("error");
    if (res.ok) {
        await __NEXTAUTH._getSession({ event: "storage" });
    }
    return {
        error,
        status: res.status,
        ok: res.ok,
        url: error ? null : data.url,
    };
}
/**
 * Signs the user out, by removing the session cookie.
 * Automatically adds the CSRF token to the request.
 *
 * [Documentation](https://next-auth.js.org/getting-started/client#signout)
 */
export async function signOut(options) {
    var _a, _b;
    const { callbackUrl = window.location.href } = options !== null && options !== void 0 ? options : {};
    const baseUrl = apiBaseUrl(__NEXTAUTH);
    const fetchOptions = {
        method: "post",
        headers: {
            "Content-Type": "application/x-www-form-urlencoded",
        },
        // @ts-expect-error
        body: new URLSearchParams({
            csrfToken: await getCsrfToken(),
            callbackUrl,
            json: true,
        }),
    };
    const res = await fetch(`${baseUrl}/signout`, fetchOptions);
    const data = await res.json();
    broadcast.post({ event: "session", data: { trigger: "signout" } });
    if ((_a = options === null || options === void 0 ? void 0 : options.redirect) !== null && _a !== void 0 ? _a : true) {
        const url = (_b = data.url) !== null && _b !== void 0 ? _b : callbackUrl;
        window.location.replace(url);
        // If url contains a hash, the browser does not reload the page. We reload manually
        if (url.includes("#"))
            window.location.reload();
        // @ts-expect-error
        return;
    }
    await __NEXTAUTH._getSession({ event: "storage" });
    return data;
}
/**
 * Provider to wrap the app in to make session data available globally.
 * Can also be used to throttle the number of requests to the endpoint
 * `/api/auth/session`.
 *
 * [Documentation](https://next-auth.js.org/getting-started/client#sessionprovider)
 */
export function SessionProvider(props) {
    const { children, baseUrl, basePath, staleTime = 0 } = props;
    if (baseUrl)
        __NEXTAUTH.baseUrl = baseUrl;
    if (basePath)
        __NEXTAUTH.basePath = basePath;
    /**
     * If session was `null`, there was an attempt to fetch it,
     * but it failed, but we still treat it as a valid initial value.
     */
    const hasInitialSession = props.session !== undefined;
    /** If session was passed, initialize as already synced */
    __NEXTAUTH._lastSync = hasInitialSession ? now() : 0;
    const [session, setSession] = React.useState(() => {
        if (hasInitialSession)
            __NEXTAUTH._session = props.session;
        return props.session;
    });
    /** If session was passed, initialize as not loading */
    const [loading, setLoading] = React.useState(!hasInitialSession);
    React.useEffect(() => {
        __NEXTAUTH._getSession = async ({ event } = {}) => {
            try {
                const storageEvent = event === "storage";
                // We should always update if we don't have a client session yet
                // or if there are events from other tabs/windows
                if (storageEvent || __NEXTAUTH._session === undefined) {
                    __NEXTAUTH._lastSync = now();
                    __NEXTAUTH._session = await getSession({
                        broadcast: !storageEvent,
                    });
                    setSession(__NEXTAUTH._session);
                    return;
                }
                if (
                // If there is no time defined for when a session should be considered
                // stale, then it's okay to use the value we have until an event is
                // triggered which updates it
                (staleTime === 0 && !event) ||
                    // If the client doesn't have a session then we don't need to call
                    // the server to check if it does (if they have signed in via another
                    // tab or window that will come through as a "stroage" event
                    // event anyway)
                    (staleTime > 0 && __NEXTAUTH._session === null) ||
                    // Bail out early if the client session is not stale yet
                    (staleTime > 0 && now() < __NEXTAUTH._lastSync + staleTime)) {
                    return;
                }
                // An event or session staleness occurred, update the client session.
                __NEXTAUTH._lastSync = now();
                __NEXTAUTH._session = await getSession();
                setSession(__NEXTAUTH._session);
            }
            catch (error) {
                logger.error("CLIENT_SESSION_ERROR", error);
            }
            finally {
                setLoading(false);
            }
        };
        __NEXTAUTH._getSession();
    }, [staleTime]);
    React.useEffect(() => {
        // Listen for storage events and update session if event fired from
        // another window (but suppress firing another event to avoid a loop)
        // Fetch new session data but tell it to not to fire another event to
        // avoid an infinite loop.
        // Note: We could pass session data through and do something like
        // `setData(message.data)` but that can cause problems depending
        // on how the session object is being used in the client; it is
        // more robust to have each window/tab fetch it's own copy of the
        // session object rather than share it across instances.
        const unsubscribe = broadcast.receive(() => __NEXTAUTH._getSession({ event: "storage" }));
        return () => unsubscribe();
    }, []);
    React.useEffect(() => {
        // Set up visibility change
        // Listen for document visibility change events and
        // if visibility of the document changes, re-fetch the session.
        const visibilityHandler = () => {
            !document.hidden && __NEXTAUTH._getSession({ event: "visibilitychange" });
        };
        document.addEventListener("visibilitychange", visibilityHandler, false);
        return () => document.removeEventListener("visibilitychange", visibilityHandler, false);
    }, []);
    React.useEffect(() => {
        const { refetchInterval } = props;
        // Set up polling
        if (refetchInterval) {
            const refetchIntervalTimer = setInterval(() => {
                if (__NEXTAUTH._session) {
                    __NEXTAUTH._getSession({ event: "poll" });
                }
            }, refetchInterval * 1000);
            return () => clearInterval(refetchIntervalTimer);
        }
    }, [props.refetchInterval]);
    const value = React.useMemo(() => ({
        data: session,
        status: loading
            ? "loading"
            : session
                ? "authenticated"
                : "unauthenticated",
    }), [session, loading]);
    return (_jsx(SessionContext.Provider, Object.assign({ value: value }, { children: children }), void 0));
}
